@@ -26,21 +26,12 @@ import collections
 import uuid
 
 from oslo_config import cfg
-import six
 
 from oslo_messaging._drivers import common as rpc_common
 
-deprecated_durable_opts = [
-    cfg.DeprecatedOpt('amqp_durable_queues',
-                      group='DEFAULT'),
-    cfg.DeprecatedOpt('rabbit_durable_queues',
-                      group='DEFAULT')
-]
-
 amqp_opts = [
     cfg.BoolOpt('amqp_durable_queues',
                 default=False,
-                deprecated_opts=deprecated_durable_opts,
                 help='Use durable queues in AMQP.'),
     cfg.BoolOpt('amqp_auto_delete',
                 default=False,
@@ -70,7 +61,7 @@ def unpack_context(msg):
     """Unpack context from msg."""
     context_dict = {}
     for key in list(msg.keys()):
-        key = six.text_type(key)
+        key = str(key)
         if key.startswith('_context_'):
             value = msg.pop(key)
             context_dict[key[9:]] = value
