@@ -27,26 +27,24 @@ functions scheduled by the Controller.
 import abc
 import collections
 import logging
-from monotonic import monotonic as now  # noqa
 import os
 import platform
+import queue
 import random
 import sys
 import threading
+import time
 import uuid
 
+from oslo_utils import eventletutils
 import proton
 import pyngus
-from six import iteritems
-from six import itervalues
-from six import moves
 
 from oslo_messaging._drivers.amqp1_driver.addressing import AddresserFactory
 from oslo_messaging._drivers.amqp1_driver.addressing import keyify
 from oslo_messaging._drivers.amqp1_driver.addressing import SERVICE_NOTIFY
 from oslo_messaging._drivers.amqp1_driver.addressing import SERVICE_RPC
 from oslo_messaging._drivers.amqp1_driver import eventloop
-from oslo_messaging._i18n import _LE, _LI, _LW
 from oslo_messaging import exceptions
 from oslo_messaging.target import Target
 from oslo_messaging import transport
@@ -81,7 +79,7 @@ class SubscribeTask(Task):
         self._subscriber_id = listener.id
         self._in_queue = listener.incoming
         self._service = SERVICE_NOTIFY if notifications else SERVICE_RPC
-        self._wakeup = threading.Event()
+        self._wakeup = eventletutils.Event()
 
     def wait(self):
         self._wakeup.wait()
@@ -108,7 +106,7 @@ class SendTask(Task):
         self.service = SERVICE_NOTIFY if notification else SERVICE_RPC
         self.timer = None
         self._retry = None if retry is None or retry < 0 else retry
-        self._wakeup = threading.Event()
+        self._wakeup = eventletutils.Event()
         self._error = None
         self._sender = None
 
@@ -306,7 +304,7 @@ class MessageDispositionTask(Task):
             self._disposition(self._released)
         except Exception as e:
             # there's really nothing we can do about a failed disposition.
-            LOG.exception(_LE("Message acknowledgment failed: %s"), e)
+            LOG.exception("Message acknowledgment failed: %s", e)
 
 
 class Sender(pyngus.SenderEventHandler):
@@ -413,8 +411,8 @@ class Sender(pyngus.SenderEventHandler):
     def sender_remote_closed(self, sender_link, pn_condition):
         # The remote has initiated a close.  This could happen when the message
         # bus is shutting down, or it detected an error
-        LOG.warning(_LW("sender %(addr)s failed due to remote initiated close:"
-                        " condition=%(cond)s"),
+        LOG.warning("Sender %(addr)s failed due to remote initiated close:"
+                    " condition=%(cond)s",
                     {'addr': self._address, 'cond': pn_condition})
         self._link.close()
         # sender_closed() will be called once the link completes closing
@@ -424,7 +422,7 @@ class Sender(pyngus.SenderEventHandler):
 
     def sender_failed(self, sender_link, error):
         """Protocol error occurred."""
-        LOG.warning(_LW("sender %(addr)s failed error=%(error)s"),
+        LOG.warning("Sender %(addr)s failed error=%(error)s",
                     {'addr': self._address, 'error': error})
         self._handle_sender_closed(str(error))
 
@@ -486,8 +484,8 @@ class Sender(pyngus.SenderEventHandler):
                 self._unacked.discard(send_task)
                 if state == Sender._ACCEPTED:
                     send_task._on_ack(Sender._ACCEPTED, info)
-                elif (state == Sender._RELEASED
-                      or (state == Sender._MODIFIED and
+                elif (state == Sender._RELEASED or
+                      (state == Sender._MODIFIED and
                           # assuming delivery-failed means in-doubt:
                           not info.get("delivery-failed") and
                           not info.get("undeliverable-here"))):
@@ -634,13 +632,13 @@ class Replies(pyngus.ReceiverEventHandler):
         receiver link has initiated closing the connection.
         """
         if pn_condition:
-            LOG.error(_LE("Reply subscription closed by peer: %s"),
+            LOG.error("Reply subscription closed by peer: %s",
                       pn_condition)
         receiver.close()
 
     def receiver_failed(self, receiver_link, error):
         """Protocol error occurred."""
-        LOG.error(_LE("Link to reply queue failed. error=%(error)s"),
+        LOG.error("Link to reply queue failed. error=%(error)s",
                   {"error": error})
         self._on_down()
 
@@ -656,8 +654,8 @@ class Replies(pyngus.ReceiverEventHandler):
             self._correlation[key](message)
             receiver.message_accepted(handle)
         except KeyError:
-            LOG.warning(_LW("Can't find receiver for response msg id=%s, "
-                            "dropping!"), key)
+            LOG.warning("Can't find receiver for response msg id=%s, "
+                        "dropping!", key)
             receiver.message_modified(handle, True, True, None)
         # ensure we have enough credit
         if receiver.capacity <= self._capacity_low:
@@ -723,13 +721,13 @@ class Server(pyngus.ReceiverEventHandler):
                 "addr": receiver.source_address or receiver.target_address,
                 "err_msg": pn_condition
             }
-            LOG.error(_LE("Server subscription %(addr)s closed "
-                          "by peer: %(err_msg)s"), vals)
+            LOG.error("Server subscription %(addr)s closed "
+                      "by peer: %(err_msg)s", vals)
         receiver.close()
 
     def receiver_failed(self, receiver_link, error):
         """Protocol error occurred."""
-        LOG.error(_LE("Listener link queue failed. error=%(error)s"),
+        LOG.error("Listener link queue failed. error=%(error)s",
                   {"error": error})
         self.receiver_closed(receiver_link)
 
@@ -819,13 +817,10 @@ class NotificationServer(Server):
 
 class Hosts(object):
     """An order list of TransportHost addresses. Connection failover progresses
-    from one host to the next.  username, password , and realm come from the
-    configuration and are used only if no username/password/realm is present in
-    the URL.
+    from one host to the next.  The default realm comes from the configuration
+    and is only used if no realm is present in the URL.
     """
-    def __init__(self, url, default_username=None,
-                 default_password=None,
-                 default_realm=None):
+    def __init__(self, url, default_realm=None):
         self.virtual_host = url.virtual_host
         if url.hosts:
             self._entries = url.hosts[:]
@@ -834,8 +829,8 @@ class Hosts(object):
                                                      port=5672)]
         for entry in self._entries:
             entry.port = entry.port or 5672
-            entry.username = entry.username or default_username
-            entry.password = entry.password or default_password
+            entry.username = entry.username
+            entry.password = entry.password
             if default_realm and entry.username and '@' not in entry.username:
                 entry.username = entry.username + '@' + default_realm
         self._current = random.randint(0, len(self._entries) - 1)  # nosec
@@ -871,7 +866,7 @@ class Controller(pyngus.ConnectionEventHandler):
         self._command = os.path.basename(sys.argv[0])
         self._pid = os.getpid()
         # queue of drivertask objects to execute on the eventloop thread
-        self._tasks = moves.queue.Queue(maxsize=500)
+        self._tasks = queue.Queue(maxsize=500)
         # limit the number of Task()'s to execute per call to _process_tasks().
         # This allows the eventloop main thread to return to servicing socket
         # I/O in a timely manner
@@ -894,16 +889,12 @@ class Controller(pyngus.ConnectionEventHandler):
         self.ssl_cert_file = config.oslo_messaging_amqp.ssl_cert_file
         self.ssl_key_file = config.oslo_messaging_amqp.ssl_key_file
         self.ssl_key_password = config.oslo_messaging_amqp.ssl_key_password
-        self.ssl_allow_insecure = \
-            config.oslo_messaging_amqp.allow_insecure_clients
         self.ssl_verify_vhost = config.oslo_messaging_amqp.ssl_verify_vhost
         self.pseudo_vhost = config.oslo_messaging_amqp.pseudo_vhost
         self.sasl_mechanisms = config.oslo_messaging_amqp.sasl_mechanisms
         self.sasl_config_dir = config.oslo_messaging_amqp.sasl_config_dir
         self.sasl_config_name = config.oslo_messaging_amqp.sasl_config_name
-        self.hosts = Hosts(url, config.oslo_messaging_amqp.username,
-                           config.oslo_messaging_amqp.password,
-                           config.oslo_messaging_amqp.sasl_default_realm)
+        self.hosts = Hosts(url, config.oslo_messaging_amqp.sasl_default_realm)
         self.conn_retry_interval = \
             config.oslo_messaging_amqp.connection_retry_interval
         self.conn_retry_backoff = \
@@ -962,13 +953,13 @@ class Controller(pyngus.ConnectionEventHandler):
 
     def shutdown(self, timeout=30):
         """Shutdown the messaging service."""
-        LOG.info(_LI("Shutting down the AMQP 1.0 connection"))
+        LOG.info("Shutting down the AMQP 1.0 connection")
         if self.processor:
             self.processor.wakeup(self._start_shutdown)
             LOG.debug("Waiting for eventloop to exit")
             self.processor.join(timeout)
             self._hard_reset("Shutting down")
-            for sender in itervalues(self._all_senders):
+            for sender in self._all_senders.values():
                 sender.destroy()
             self._all_senders.clear()
             self._servers.clear()
@@ -982,7 +973,7 @@ class Controller(pyngus.ConnectionEventHandler):
     # methods executed by Tasks created by the driver:
 
     def send(self, send_task):
-        if send_task.deadline and send_task.deadline <= now():
+        if send_task.deadline and send_task.deadline <= time.monotonic():
             send_task._on_timeout()
             return
         key = keyify(send_task.target, send_task.service)
@@ -1094,7 +1085,7 @@ class Controller(pyngus.ConnectionEventHandler):
             try:
                 self._tasks.get(False)._execute(self)
             except Exception as e:
-                LOG.exception(_LE("Error processing task: %s"), e)
+                LOG.exception("Error processing task: %s", e)
             count += 1
 
         # if we hit _max_task_batch, resume task processing later:
@@ -1135,13 +1126,13 @@ class Controller(pyngus.ConnectionEventHandler):
         point, we are ready to receive messages, so start all pending RPC
         requests.
         """
-        LOG.info(_LI("Messaging is active (%(hostname)s:%(port)s%(vhost)s)"),
+        LOG.info("Messaging is active (%(hostname)s:%(port)s%(vhost)s)",
                  {'hostname': self.hosts.current.hostname,
                   'port': self.hosts.current.port,
                   'vhost': ("/" + self.hosts.virtual_host
                             if self.hosts.virtual_host else "")})
 
-        for sender in itervalues(self._all_senders):
+        for sender in self._all_senders.values():
             sender.attach(self._socket_connection.pyngus_conn,
                           self.reply_link, self.addresser)
 
@@ -1158,7 +1149,7 @@ class Controller(pyngus.ConnectionEventHandler):
 
     def socket_error(self, error):
         """Called by eventloop when a socket error occurs."""
-        LOG.error(_LE("Socket failure: %s"), error)
+        LOG.error("Socket failure: %s", error)
         self._handle_connection_loss(str(error))
 
     # Pyngus connection event callbacks (and their helpers), all invoked from
@@ -1188,8 +1179,8 @@ class Controller(pyngus.ConnectionEventHandler):
         self.addresser = self.addresser_factory(props,
                                                 self.hosts.virtual_host
                                                 if self.pseudo_vhost else None)
-        for servers in itervalues(self._servers):
-            for server in itervalues(servers):
+        for servers in self._servers.values():
+            for server in servers.values():
                 server.attach(self._socket_connection.pyngus_conn,
                               self.addresser)
         self.reply_link = Replies(self._socket_connection.pyngus_conn,
@@ -1219,7 +1210,7 @@ class Controller(pyngus.ConnectionEventHandler):
         # connection. Acknowledge the close, and try to reconnect/failover
         # later once the connection has closed (connection_closed is called).
         if reason:
-            LOG.info(_LI("Connection closed by peer: %s"), reason)
+            LOG.info("Connection closed by peer: %s", reason)
         self._detach_senders()
         self._detach_servers()
         self.reply_link.detach()
@@ -1232,8 +1223,8 @@ class Controller(pyngus.ConnectionEventHandler):
         """
         if outcome == proton.SASL.OK:
             return
-        LOG.error(_LE("AUTHENTICATION FAILURE: Cannot connect to "
-                      "%(hostname)s:%(port)s as user %(username)s"),
+        LOG.error("AUTHENTICATION FAILURE: Cannot connect to "
+                  "%(hostname)s:%(port)s as user %(username)s",
                   {'hostname': self.hosts.current.hostname,
                    'port': self.hosts.current.port,
                    'username': self.hosts.current.username})
@@ -1254,7 +1245,7 @@ class Controller(pyngus.ConnectionEventHandler):
             # service.  Try to re-establish the connection:
             if not self._reconnecting:
                 self._reconnecting = True
-                LOG.info(_LI("delaying reconnect attempt for %d seconds"),
+                LOG.info("Delaying reconnect attempt for %d seconds",
                          self._delay)
                 self.processor.defer(lambda: self._do_reconnect(reason),
                                      self._delay)
@@ -1272,7 +1263,7 @@ class Controller(pyngus.ConnectionEventHandler):
         if not self._closing:
             self._hard_reset(reason)
             host = self.hosts.next()
-            LOG.info(_LI("Reconnecting to: %(hostname)s:%(port)s"),
+            LOG.info("Reconnecting to: %(hostname)s:%(port)s",
                      {'hostname': host.hostname, 'port': host.port})
             self._socket_connection.connect(host)
 
@@ -1286,7 +1277,7 @@ class Controller(pyngus.ConnectionEventHandler):
         del self._purged_senders[:]
         self._active_senders.clear()
         unused = []
-        for key, sender in iteritems(self._all_senders):
+        for key, sender in self._all_senders.items():
             # clean up any sender links that no longer have messages to send
             if sender.pending_messages == 0:
                 unused.append(key)
@@ -1296,8 +1287,8 @@ class Controller(pyngus.ConnectionEventHandler):
         for key in unused:
             self._all_senders[key].destroy(reason)
             del self._all_senders[key]
-        for servers in itervalues(self._servers):
-            for server in itervalues(servers):
+        for servers in self._servers.values():
+            for server in servers.values():
                 server.reset()
         if self.reply_link:
             self.reply_link.destroy()
@@ -1307,13 +1298,13 @@ class Controller(pyngus.ConnectionEventHandler):
 
     def _detach_senders(self):
         """Close all sender links"""
-        for sender in itervalues(self._all_senders):
+        for sender in self._all_senders.values():
             sender.detach()
 
     def _detach_servers(self):
         """Close all listener links"""
-        for servers in itervalues(self._servers):
-            for server in itervalues(servers):
+        for servers in self._servers.values():
+            for server in servers.values():
                 server.detach()
 
     def _purge_sender_links(self):
@@ -1339,5 +1330,5 @@ class Controller(pyngus.ConnectionEventHandler):
     @property
     def _active(self):
         # Is the connection up
-        return (self._socket_connection
-                and self._socket_connection.pyngus_conn.active)
+        return (self._socket_connection and
+                self._socket_connection.pyngus_conn.active)
