@@ -24,8 +24,10 @@ import ssl
 import sys
 import threading
 import time
+from urllib import parse
 import uuid
 
+from amqp import exceptions as amqp_exec
 import kombu
 import kombu.connection
 import kombu.entity
@@ -34,9 +36,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import eventletutils
 from oslo_utils import importutils
-import six
-import six.moves
-from six.moves.urllib import parse
 
 import oslo_messaging
 from oslo_messaging._drivers import amqp as rpc_amqp
@@ -59,7 +58,7 @@ else:
     # we will facing an issue by trying to override the threading module.
     stdlib_threading = threading
 
-# NOTE(sileht): don't exists in py2 socket module
+# NOTE(sileht): don't exist in py2 socket module
 TCP_USER_TIMEOUT = 18
 
 rabbit_opts = [
@@ -90,12 +89,12 @@ rabbit_opts = [
                     '(valid only if SSL enabled).'),
     cfg.BoolOpt('heartbeat_in_pthread',
                 default=False,
-                help="EXPERIMENTAL: Run the health check heartbeat thread"
-                     "through a native python thread. By default if this"
-                     "option isn't provided the  health check heartbeat will"
-                     "inherit the execution model from the parent process. By"
-                     "example if the parent process have monkey patched the"
-                     "stdlib by using eventlet/greenlet then the heartbeat"
+                help="EXPERIMENTAL: Run the health check heartbeat thread "
+                     "through a native python thread. By default if this "
+                     "option isn't provided the  health check heartbeat will "
+                     "inherit the execution model from the parent process. By "
+                     "example if the parent process have monkey patched the "
+                     "stdlib by using eventlet/greenlet then the heartbeat "
                      "will be run through a green thread."),
     cfg.FloatOpt('kombu_reconnect_delay',
                  default=1.0,
@@ -146,7 +145,7 @@ rabbit_opts = [
                 'queue. If you just want to make sure that all queues (except '
                 'those with auto-generated names) are mirrored across all '
                 'nodes, run: '
-                """\"rabbitmqctl set_policy HA '^(?!amq\.).*' """
+                """\"rabbitmqctl set_policy HA '^(?!amq\\.).*' """
                 """'{"ha-mode": "all"}' \""""),
     cfg.IntOpt('rabbit_transient_queues_ttl',
                min=1,
@@ -168,12 +167,23 @@ rabbit_opts = [
                default=2,
                help='How often times during the heartbeat_timeout_threshold '
                'we check the heartbeat.'),
-    cfg.IntOpt('direct_mandatory_flag',
-               default=True,
-               help='Enable/Disable the RabbitMQ mandatory flag '
-               'for direct send. The direct send is used as reply,'
-               'so the MessageUndeliverable exception is raised'
-               ' in case the client queue does not exist.'),
+    cfg.BoolOpt('direct_mandatory_flag',
+                default=True,
+                deprecated_for_removal=True,
+                deprecated_reason='Mandatory flag no longer deactivable.',
+                help='(DEPRECATED) Enable/Disable the RabbitMQ mandatory '
+                'flag for direct send. The direct send is used as reply, '
+                'so the MessageUndeliverable exception is raised '
+                'in case the client queue does not exist.'
+                'MessageUndeliverable exception will be used to loop for a '
+                'timeout to lets a chance to sender to recover.'
+                'This flag is deprecated and it will not be possible to '
+                'deactivate this functionality anymore'),
+    cfg.BoolOpt('enable_cancel_on_failover',
+                default=False,
+                help="Enable x-cancel-on-ha-failover flag so that "
+                     "rabbitmq server will cancel and notify consumers"
+                     "when queue is down")
 ]
 
 LOG = logging.getLogger(__name__)
@@ -192,7 +202,7 @@ def _get_queue_arguments(rabbit_ha_queues, rabbit_queue_ttl):
     no longer controlled by the x-ha-policy argument when declaring a
     queue. If you just want to make sure that all queues (except those
     with auto-generated names) are mirrored across all nodes, run:
-      rabbitmqctl set_policy HA '^(?!amq\.).*' '{"ha-mode": "all"}'
+      rabbitmqctl set_policy HA '^(?!amq\\.).*' '{"ha-mode": "all"}'
 
     If the rabbit_queue_ttl option is > 0, then the queue is
     declared with the "Queue TTL" value as described here:
@@ -235,7 +245,8 @@ class Consumer(object):
 
     def __init__(self, exchange_name, queue_name, routing_key, type, durable,
                  exchange_auto_delete, queue_auto_delete, callback,
-                 nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0):
+                 nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0,
+                 enable_cancel_on_failover=False):
         """Init the Consumer class with the exchange_name, routing_key,
         type, durable auto_delete
         """
@@ -257,10 +268,16 @@ class Consumer(object):
             type=type,
             durable=self.durable,
             auto_delete=self.exchange_auto_delete)
+        self.enable_cancel_on_failover = enable_cancel_on_failover
 
     def declare(self, conn):
         """Re-declare the queue after a rabbit (re)connect."""
 
+        consumer_arguments = None
+        if self.enable_cancel_on_failover:
+            consumer_arguments = {
+                "x-cancel-on-ha-failover": True}
+
         self.queue = kombu.entity.Queue(
             name=self.queue_name,
             channel=conn.channel,
@@ -268,7 +285,9 @@ class Consumer(object):
             durable=self.durable,
             auto_delete=self.queue_auto_delete,
             routing_key=self.routing_key,
-            queue_arguments=self.queue_arguments)
+            queue_arguments=self.queue_arguments,
+            consumer_arguments=consumer_arguments
+        )
 
         try:
             LOG.debug('[%s] Queue.declare: %s',
@@ -320,14 +339,14 @@ class Consumer(object):
             self.declare(conn)
         try:
             self.queue.consume(callback=self._callback,
-                               consumer_tag=six.text_type(tag),
+                               consumer_tag=str(tag),
                                nowait=self.nowait)
         except conn.connection.channel_errors as exc:
             # We retries once because of some races that we can
             # recover before informing the deployer
-            # bugs.launchpad.net/oslo.messaging/+bug/1581148
-            # bugs.launchpad.net/oslo.messaging/+bug/1609766
-            # bugs.launchpad.net/neutron/+bug/1318721
+            # https://bugs.launchpad.net/oslo.messaging/+bug/1581148
+            # https://bugs.launchpad.net/oslo.messaging/+bug/1609766
+            # https://bugs.launchpad.net/neutron/+bug/1318721
 
             # 406 error code relates to messages that are doubled ack'd
 
@@ -340,14 +359,14 @@ class Consumer(object):
                                    exc.method_name == 'Basic.ack'):
                 self.declare(conn)
                 self.queue.consume(callback=self._callback,
-                                   consumer_tag=six.text_type(tag),
+                                   consumer_tag=str(tag),
                                    nowait=self.nowait)
             else:
                 raise
 
     def cancel(self, tag):
         LOG.trace('ConsumerBase.cancel: canceling %s', tag)
-        self.queue.cancel(six.text_type(tag))
+        self.queue.cancel(str(tag))
 
     def _callback(self, message):
         """Call callback with deserialized message.
@@ -444,8 +463,6 @@ class ConnectionLock(DummyConnectionLock):
 class Connection(object):
     """Connection object."""
 
-    pools = {}
-
     def __init__(self, conf, url, purpose):
         # NOTE(viktors): Parse config options
         driver_conf = conf.oslo_messaging_rabbit
@@ -471,6 +488,7 @@ class Connection(object):
         self.kombu_failover_strategy = driver_conf.kombu_failover_strategy
         self.kombu_compression = driver_conf.kombu_compression
         self.heartbeat_in_pthread = driver_conf.heartbeat_in_pthread
+        self.enable_cancel_on_failover = driver_conf.enable_cancel_on_failover
 
         if self.heartbeat_in_pthread:
             # NOTE(hberaud): Experimental: threading module is in use to run
@@ -498,6 +516,7 @@ class Connection(object):
             # if it was already monkey patched by eventlet/greenlet.
             global threading
             threading = stdlib_threading
+
         self.direct_mandatory_flag = driver_conf.direct_mandatory_flag
 
         if self.ssl:
@@ -528,7 +547,10 @@ class Connection(object):
                 self._url += url.virtual_host
         elif not url.hosts:
             host = oslo_messaging.transport.TransportHost('')
-            self._url = self._transform_transport_url(
+            # NOTE(moguimar): default_password in this function's context is
+            #                 a fallback option, not a hardcoded password.
+            #                 username and password are read from host.
+            self._url = self._transform_transport_url(  # nosec
                 url, host, default_username='guest', default_password='guest',
                 default_hostname='localhost')
 
@@ -658,7 +680,9 @@ class Connection(object):
         except KeyError:
             raise RuntimeError("Invalid SSL version : %s" % version)
 
-    def _transform_transport_url(self, url, host, default_username='',
+    # NOTE(moguimar): default_password in this function's context is just
+    #                 a fallback option, not a hardcoded password.
+    def _transform_transport_url(self, url, host, default_username='',  # nosec
                                  default_password='', default_hostname=''):
         transport = url.transport.replace('kombu+', '')
         transport = transport.replace('rabbit', 'amqp')
@@ -755,7 +779,7 @@ class Connection(object):
             info = {'err_str': exc, 'sleep_time': interval}
             info.update(self._get_connection_info(conn_error=True))
 
-            if 'Socket closed' in six.text_type(exc):
+            if 'Socket closed' in str(exc):
                 LOG.error('[%(connection_id)s] AMQP server'
                           ' %(hostname)s:%(port)s closed'
                           ' the connection. Check login credentials:'
@@ -869,8 +893,8 @@ class Connection(object):
         """Close/release this connection."""
         self._heartbeat_stop()
         if self.connection:
-            for consumer in six.moves.filter(lambda c: c.type == 'fanout',
-                                             self._consumers):
+            for consumer in filter(lambda c: c.type == 'fanout',
+                                   self._consumers):
                 LOG.debug('[connection close] Deleting fanout '
                           'queue: %s ' % consumer.queue.name)
                 consumer.queue.delete()
@@ -979,13 +1003,6 @@ class Connection(object):
     def _heartbeat_thread_job(self):
         """Thread that maintains inactive connections
         """
-        # NOTE(hberaud): Python2 doesn't have ConnectionRefusedError
-        # defined so to switch connections destination on failure
-        # with python2 and python3 we need to wrapp adapt connection refused
-        try:
-            ConnectRefuseError = ConnectionRefusedError
-        except NameError:
-            ConnectRefuseError = socket.error
 
         while not self._heartbeat_exit_event.is_set():
             with self._connection_lock.for_heartbeat():
@@ -1012,9 +1029,10 @@ class Connection(object):
                     # ensure_connection for switching the
                     # connection destination.
                     except (socket.timeout,
-                            ConnectRefuseError,
+                            ConnectionRefusedError,
                             OSError,
-                            kombu.exceptions.OperationalError) as exc:
+                            kombu.exceptions.OperationalError,
+                            amqp_exec.ConnectionForced) as exc:
                         LOG.info("A recoverable connection/channel error "
                                  "occurred, trying to reconnect: %s", exc)
                         self.ensure_connection()
@@ -1120,31 +1138,35 @@ class Connection(object):
         responses for call/multicall
         """
 
-        consumer = Consumer(exchange_name='',  # using default exchange
-                            queue_name=topic,
-                            routing_key='',
-                            type='direct',
-                            durable=False,
-                            exchange_auto_delete=False,
-                            queue_auto_delete=False,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues,
-                            rabbit_queue_ttl=self.rabbit_transient_queues_ttl)
+        consumer = Consumer(
+            exchange_name='',  # using default exchange
+            queue_name=topic,
+            routing_key='',
+            type='direct',
+            durable=False,
+            exchange_auto_delete=False,
+            queue_auto_delete=False,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            rabbit_queue_ttl=self.rabbit_transient_queues_ttl,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
     def declare_topic_consumer(self, exchange_name, topic, callback=None,
                                queue_name=None):
         """Create a 'topic' consumer."""
-        consumer = Consumer(exchange_name=exchange_name,
-                            queue_name=queue_name or topic,
-                            routing_key=topic,
-                            type='topic',
-                            durable=self.amqp_durable_queues,
-                            exchange_auto_delete=self.amqp_auto_delete,
-                            queue_auto_delete=self.amqp_auto_delete,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues)
+        consumer = Consumer(
+            exchange_name=exchange_name,
+            queue_name=queue_name or topic,
+            routing_key=topic,
+            type='topic',
+            durable=self.amqp_durable_queues,
+            exchange_auto_delete=self.amqp_auto_delete,
+            queue_auto_delete=self.amqp_auto_delete,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
@@ -1155,16 +1177,18 @@ class Connection(object):
         exchange_name = '%s_fanout' % topic
         queue_name = '%s_fanout_%s' % (topic, unique)
 
-        consumer = Consumer(exchange_name=exchange_name,
-                            queue_name=queue_name,
-                            routing_key=topic,
-                            type='fanout',
-                            durable=False,
-                            exchange_auto_delete=True,
-                            queue_auto_delete=False,
-                            callback=callback,
-                            rabbit_ha_queues=self.rabbit_ha_queues,
-                            rabbit_queue_ttl=self.rabbit_transient_queues_ttl)
+        consumer = Consumer(
+            exchange_name=exchange_name,
+            queue_name=queue_name,
+            routing_key=topic,
+            type='fanout',
+            durable=False,
+            exchange_auto_delete=True,
+            queue_auto_delete=False,
+            callback=callback,
+            rabbit_ha_queues=self.rabbit_ha_queues,
+            rabbit_queue_ttl=self.rabbit_transient_queues_ttl,
+            enable_cancel_on_failover=self.enable_cancel_on_failover)
 
         self.declare_consumer(consumer)
 
@@ -1206,8 +1230,8 @@ class Connection(object):
         """Publish a message."""
 
         if not (exchange.passive or exchange.name in self._declared_exchanges):
-                exchange(self.channel).declare()
-                self._declared_exchanges.add(exchange.name)
+            exchange(self.channel).declare()
+            self._declared_exchanges.add(exchange.name)
 
         log_info = {'msg': msg,
                     'who': exchange or 'default',
@@ -1288,7 +1312,7 @@ class Connection(object):
                 # the 404 kombu ChannelError and retry until the exchange
                 # appears
                 raise rpc_amqp.AMQPDestinationNotFound(
-                    "exchange %s doesn't exists" % exchange.name)
+                    "exchange %s doesn't exist" % exchange.name)
             raise
 
     def direct_send(self, msg_id, msg):
@@ -1370,6 +1394,10 @@ class RabbitDriver(amqpdriver.AMQPDriverBase):
         # the pool configuration properties
         max_size = conf.oslo_messaging_rabbit.rpc_conn_pool_size
         min_size = conf.oslo_messaging_rabbit.conn_pool_min_size
+        if max_size < min_size:
+            raise RuntimeError(
+                f"rpc_conn_pool_size: {max_size} must be greater than "
+                f"or equal to conn_pool_min_size: {min_size}")
         ttl = conf.oslo_messaging_rabbit.conn_pool_ttl
 
         connection_pool = pool.ConnectionPool(
