@@ -14,9 +14,7 @@
 #    under the License.
 
 __all__ = ['MessagingException', 'MessagingTimeout', 'MessageDeliveryFailure',
-           'InvalidTarget']
-
-import six
+           'InvalidTarget', 'MessageUndeliverable']
 
 
 class MessagingException(Exception):
@@ -35,6 +33,17 @@ class InvalidTarget(MessagingException, ValueError):
     """Raised if a target does not meet certain pre-conditions."""
 
     def __init__(self, msg, target):
-        msg = msg + ":" + six.text_type(target)
+        msg = msg + ":" + str(target)
         super(InvalidTarget, self).__init__(msg)
         self.target = target
+
+
+class MessageUndeliverable(Exception):
+    """Raised if message is not routed with mandatory flag"""
+
+    def __init__(self, exception, exchange, routing_key, message):
+        super(MessageUndeliverable, self).__init__()
+        self.exception = exception
+        self.exchange = exchange
+        self.routing_key = routing_key
+        self.message = message
