@@ -22,11 +22,9 @@ import uuid
 
 from oslo_config import cfg
 from oslo_utils import timeutils
-import six
 from stevedore import extension
 from stevedore import named
 
-from oslo_messaging._i18n import _LE
 from oslo_messaging import serializer as msg_serializer
 from oslo_messaging import transport as msg_transport
 
@@ -95,7 +93,8 @@ def _send_notification():
                             'oslo.messaging.notify.drivers'
                         ).names(),
                         help='Notification driver')
-    parser.add_argument('payload')
+    parser.add_argument('payload',
+                        help="the notification payload (dict)")
     args = parser.parse_args()
     conf = cfg.ConfigOpts()
     conf([],
@@ -106,8 +105,7 @@ def _send_notification():
     notifier._notify({}, args.event_type, args.payload, args.priority)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Driver(object):
+class Driver(object, metaclass=abc.ABCMeta):
     """Base driver for Notifications"""
 
     def __init__(self, conf, topics, transport):
@@ -182,7 +180,7 @@ class Notifier(object):
 
     Notification messages follow the following format::
 
-        {'message_id': six.text_type(uuid.uuid4()),
+        {'message_id': str(uuid.uuid4()),
          'publisher_id': 'compute.host1',
          'timestamp': timeutils.utcnow(),
          'priority': 'WARN',
@@ -300,20 +298,20 @@ class Notifier(object):
         payload = self._serializer.serialize_entity(ctxt, payload)
         ctxt = self._serializer.serialize_context(ctxt)
 
-        msg = dict(message_id=six.text_type(uuid.uuid4()),
+        msg = dict(message_id=str(uuid.uuid4()),
                    publisher_id=publisher_id or self.publisher_id,
                    event_type=event_type,
                    priority=priority,
                    payload=payload,
-                   timestamp=six.text_type(timeutils.utcnow()))
+                   timestamp=str(timeutils.utcnow()))
 
         def do_notify(ext):
             try:
                 ext.obj.notify(ctxt, msg, priority, retry or self.retry)
             except Exception as e:
-                _LOG.exception(_LE("Problem '%(e)s' attempting to send to "
-                                   "notification system. Payload=%(payload)s"),
-                               dict(e=e, payload=payload))
+                _LOG.exception("Problem '%(e)s' attempting to send to "
+                               "notification system. Payload=%(payload)s",
+                               {'e': e, 'payload': payload})
 
         if self._driver_mgr.extensions:
             self._driver_mgr.map(do_notify)
