@@ -12,21 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-__all__ = ['Serializer', 'NoOpSerializer', 'JsonPayloadSerializer',
-           'RequestContextSerializer']
-
 """Provides the definition of a message serialization handler"""
 
 import abc
 
-from debtcollector import removals
-from oslo_context import context as common_context
 from oslo_serialization import jsonutils
-import six
+
+__all__ = ['Serializer', 'NoOpSerializer', 'JsonPayloadSerializer']
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Serializer(object):
+class Serializer(object, metaclass=abc.ABCMeta):
     """Generic (de-)serialization definition base class."""
 
     @abc.abstractmethod
@@ -64,31 +59,6 @@ class Serializer(object):
         """
 
 
-@removals.removed_class("RequestContextSerializer",
-                        version="4.6",
-                        removal_version="5.0")
-class RequestContextSerializer(Serializer):
-
-    def __init__(self, base):
-        self._base = base
-
-    def serialize_entity(self, context, entity):
-        if not self._base:
-            return entity
-        return self._base.serialize_entity(context, entity)
-
-    def deserialize_entity(self, context, entity):
-        if not self._base:
-            return entity
-        return self._base.deserialize_entity(context, entity)
-
-    def serialize_context(self, context):
-        return context.to_dict()
-
-    def deserialize_context(self, context):
-        return common_context.RequestContext.from_dict(context)
-
-
 class NoOpSerializer(Serializer):
     """A serializer that does nothing."""
 
