@@ -15,20 +15,21 @@
 import copy
 import logging
 import os
+import queue
 import select
 import shlex
 import shutil
-from six.moves import mock
 import socket
 import subprocess
 import sys
 import tempfile
 import threading
 import time
+from unittest import mock
 import uuid
 
+from oslo_utils import eventletutils
 from oslo_utils import importutils
-from six import moves
 from string import Template
 import testtools
 
@@ -51,8 +52,8 @@ if pyngus:
 # The Cyrus-based SASL tests can only be run if the installed version of proton
 # has been built with Cyrus SASL support.
 _proton = importutils.try_import("proton")
-CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton
-                 and getattr(_proton.SASL, "extended", lambda: False)())
+CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton and
+                 getattr(_proton.SASL, "extended", lambda: False)())
 # same with SSL
 # SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
 SSL_ENABLED = False
@@ -73,10 +74,10 @@ class _ListenerThread(threading.Thread):
         self.listener = listener
         self.msg_count = msg_count
         self._msg_ack = msg_ack
-        self.messages = moves.queue.Queue()
+        self.messages = queue.Queue()
         self.daemon = True
-        self.started = threading.Event()
-        self._done = threading.Event()
+        self.started = eventletutils.Event()
+        self._done = eventletutils.Event()
         self.start()
         self.started.wait()
 
@@ -105,7 +106,7 @@ class _ListenerThread(threading.Thread):
             while True:
                 m = self.messages.get(False)
                 msgs.append(m)
-        except moves.queue.Empty:
+        except queue.Empty:
             pass
         return msgs
 
@@ -904,17 +905,6 @@ mech_list: ${mechs}
                           addr,
                           retry=0)
 
-    def test_authentication_default_username(self):
-        """Verify that a configured username/password is used if none appears
-        in the URL.
-        Deprecated: username password deprecated in favor of transport_url
-        """
-        addr = "amqp://%s:%d" % (self._broker.host, self._broker.port)
-        self.config(username="joe@myrealm",
-                    password="secret",
-                    group="oslo_messaging_amqp")
-        self._authentication_test(addr)
-
     def test_authentication_default_realm(self):
         """Verify that default realm is used if none present in username"""
         addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
@@ -1157,7 +1147,7 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
         self._addrs = {'unicast.test-topic': 2,
                        'broadcast.test-topic.all': 2,
                        'exclusive.test-topic.server': 2}
-        self._recovered = threading.Event()
+        self._recovered = eventletutils.Event()
         self._count = 0
 
         def _on_active(link):
@@ -1316,10 +1306,10 @@ class TestAddressing(test_utils.BaseTestCase):
         s1_payload = [m.message.get('msg') for m in rl[0].get_messages()]
         s2_payload = [m.message.get('msg') for m in rl[1].get_messages()]
 
-        self.assertTrue("Server1" in s1_payload
-                        and "Server2" not in s1_payload)
-        self.assertTrue("Server2" in s2_payload
-                        and "Server1" not in s2_payload)
+        self.assertTrue("Server1" in s1_payload and
+                        "Server2" not in s1_payload)
+        self.assertTrue("Server2" in s2_payload and
+                        "Server1" not in s2_payload)
         self.assertEqual(s1_payload.count("Fanout"), 1)
         self.assertEqual(s2_payload.count("Fanout"), 1)
         self.assertEqual((s1_payload + s2_payload).count("Anycast1"), 1)
@@ -2111,7 +2101,7 @@ class FakeBroker(threading.Thread):
 
         self._connections = {}
         self._sources = {}
-        self._pause = threading.Event()
+        self._pause = eventletutils.Event()
         # count of messages forwarded, by messaging pattern
         self.direct_count = 0
         self.topic_count = 0
