@@ -17,7 +17,6 @@ import uuid
 
 import concurrent.futures
 from oslo_config import cfg
-import six.moves
 from testtools import matchers
 
 import oslo_messaging
@@ -28,7 +27,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def setUp(self):
         super(CallTestCase, self).setUp(conf=cfg.ConfigOpts())
-        if self.url.startswith("kafka://"):
+        if self.rpc_url.startswith("kafka://"):
             self.skipTest("kafka does not support RPC API")
 
         self.conf.prog = "test_prog"
@@ -39,7 +38,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def test_specific_server(self):
         group = self.useFixture(utils.RpcServerGroupFixture(
-            self.conf, self.url)
+            self.conf, self.rpc_url)
         )
         client = group.client(1)
         client.append(text='open')
@@ -55,7 +54,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def test_server_in_group(self):
         group = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url)
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url)
         )
 
         client = group.client()
@@ -73,13 +72,13 @@ class CallTestCase(utils.SkipIfNoTransportURL):
         # teardown may hang unless we broadcast all control messages
         # to each server
         group1 = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url,
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url,
                                         use_fanout_ctrl=True))
         group2 = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url, exchange="a",
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url, exchange="a",
                                         use_fanout_ctrl=True))
         group3 = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url, exchange="b",
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url, exchange="b",
                                         use_fanout_ctrl=True))
 
         client1 = group1.client(1)
@@ -113,7 +112,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def test_timeout(self):
         transport = self.useFixture(
-            utils.RPCTransportFixture(self.conf, self.url)
+            utils.RPCTransportFixture(self.conf, self.rpc_url)
         )
         target = oslo_messaging.Target(topic="no_such_topic")
         c = utils.ClientStub(transport.transport, target, timeout=1)
@@ -122,7 +121,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def test_exception(self):
         group = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url)
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url)
         )
         client = group.client(1)
         client.add(increment=2)
@@ -130,12 +129,12 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
     def test_timeout_with_concurrently_queues(self):
         transport = self.useFixture(
-            utils.RPCTransportFixture(self.conf, self.url)
+            utils.RPCTransportFixture(self.conf, self.rpc_url)
         )
         target = oslo_messaging.Target(topic="topic_" + str(uuid.uuid4()),
                                        server="server_" + str(uuid.uuid4()))
         server = self.useFixture(
-            utils.RpcServerFixture(self.conf, self.url, target,
+            utils.RpcServerFixture(self.conf, self.rpc_url, target,
                                    executor="threading"))
         client = utils.ClientStub(transport.transport, target,
                                   cast=False, timeout=5)
@@ -152,13 +151,47 @@ class CallTestCase(utils.SkipIfNoTransportURL):
 
         self.assertEqual(10, server.endpoint.ival)
 
-    def test_monitor_long_call(self):
-        if not (self.url.startswith("rabbit://")
-                or self.url.startswith("amqp://")):
+    def test_mandatory_call(self):
+        if not self.rpc_url.startswith("rabbit://"):
             self.skipTest("backend does not support call monitoring")
 
         transport = self.useFixture(utils.RPCTransportFixture(self.conf,
-                                                              self.url))
+                                                              self.rpc_url))
+        target = oslo_messaging.Target(topic='topic_' + str(uuid.uuid4()),
+                                       server='server_' + str(uuid.uuid4()))
+
+        # test for mandatory flag using transport-options, see:
+        # https://blueprints.launchpad.net/oslo.messaging/+spec/transport-options
+        # first test with `at_least_once=False` raises a "MessagingTimeout"
+        # error since there is no control if the queue actually exists.
+        # (Default behavior)
+        options = oslo_messaging.TransportOptions(at_least_once=False)
+        client1 = utils.ClientStub(transport.transport, target,
+                                   cast=False, timeout=1,
+                                   transport_options=options)
+
+        self.assertRaises(oslo_messaging.MessagingTimeout,
+                          client1.delay)
+
+        # second test with `at_least_once=True` raises a "MessageUndeliverable"
+        # caused by mandatory flag.
+        # the MessageUndeliverable error is raised immediately without waiting
+        # any timeout
+        options2 = oslo_messaging.TransportOptions(at_least_once=True)
+        client2 = utils.ClientStub(transport.transport, target,
+                                   cast=False, timeout=60,
+                                   transport_options=options2)
+
+        self.assertRaises(oslo_messaging.MessageUndeliverable,
+                          client2.delay)
+
+    def test_monitor_long_call(self):
+        if not (self.rpc_url.startswith("rabbit://") or
+                self.rpc_url.startswith("amqp://")):
+            self.skipTest("backend does not support call monitoring")
+
+        transport = self.useFixture(utils.RPCTransportFixture(self.conf,
+                                                              self.rpc_url))
         target = oslo_messaging.Target(topic='topic_' + str(uuid.uuid4()),
                                        server='server_' + str(uuid.uuid4()))
 
@@ -168,7 +201,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
                 return seconds
 
         self.useFixture(
-            utils.RpcServerFixture(self.conf, self.url, target,
+            utils.RpcServerFixture(self.conf, self.rpc_url, target,
                                    executor='threading',
                                    endpoint=_endpoint()))
 
@@ -204,10 +237,10 @@ class CallTestCase(utils.SkipIfNoTransportURL):
                 return echo
 
         transport = self.useFixture(
-            utils.RPCTransportFixture(self.conf, self.url)
+            utils.RPCTransportFixture(self.conf, self.rpc_url)
         )
         self.useFixture(
-            utils.RpcServerFixture(self.conf, self.url, target,
+            utils.RpcServerFixture(self.conf, self.rpc_url, target,
                                    executor="threading",
                                    endpoint=_endpoint(target)))
         client1 = utils.ClientStub(transport.transport, target,
@@ -246,7 +279,7 @@ class CallTestCase(utils.SkipIfNoTransportURL):
         target = oslo_messaging.Target(topic="topic_" + str(uuid.uuid4()),
                                        server="server_" + str(uuid.uuid4()))
         transport = self.useFixture(
-            utils.RPCTransportFixture(self.conf, self.url)
+            utils.RPCTransportFixture(self.conf, self.rpc_url)
         )
         self.assertRaises(TypeError,
                           oslo_messaging.get_rpc_server,
@@ -263,12 +296,12 @@ class CastTestCase(utils.SkipIfNoTransportURL):
 
     def setUp(self):
         super(CastTestCase, self).setUp()
-        if self.url.startswith("kafka://"):
+        if self.rpc_url.startswith("kafka://"):
             self.skipTest("kafka does not support RPC API")
 
     def test_specific_server(self):
         group = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url)
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url)
         )
         client = group.client(1, cast=True)
         client.append(text='open')
@@ -287,10 +320,10 @@ class CastTestCase(utils.SkipIfNoTransportURL):
             self.assertEqual(0, group.servers[i].endpoint.ival)
 
     def test_server_in_group(self):
-        if self.url.startswith("amqp:"):
+        if self.rpc_url.startswith("amqp:"):
             self.skipTest("QPID-6307")
         group = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url)
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url)
         )
         client = group.client(cast=True)
         for i in range(20):
@@ -309,7 +342,7 @@ class CastTestCase(utils.SkipIfNoTransportURL):
 
     def test_fanout(self):
         group = self.useFixture(
-            utils.RpcServerGroupFixture(self.conf, self.url)
+            utils.RpcServerGroupFixture(self.conf, self.rpc_url)
         )
         client = group.client('all', cast=True)
         client.append(text='open')
@@ -328,29 +361,37 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
     # NOTE(sileht): Each test must not use the same topics
     # to be run in parallel
 
+    # NOTE(ansmith): kafka partition assignment delay requires
+    # longer timeouts for test completion
+
     def test_simple(self):
-        if self.url.startswith("kafka://"):
+        get_timeout = 1
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 5
             self.conf.set_override('consumer_group', 'test_simple',
                                    group='oslo_messaging_kafka')
 
         listener = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['test_simple']))
+            utils.NotificationFixture(self.conf, self.notify_url,
+                                      ['test_simple']))
         notifier = listener.notifier('abc')
 
         notifier.info({}, 'test', 'Hello World!')
-        event = listener.events.get(timeout=1)
+        event = listener.events.get(timeout=get_timeout)
         self.assertEqual('info', event[0])
         self.assertEqual('test', event[1])
         self.assertEqual('Hello World!', event[2])
         self.assertEqual('abc', event[3])
 
     def test_multiple_topics(self):
-        if self.url.startswith("kafka://"):
+        get_timeout = 1
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 5
             self.conf.set_override('consumer_group', 'test_multiple_topics',
                                    group='oslo_messaging_kafka')
 
         listener = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['a', 'b']))
+            utils.NotificationFixture(self.conf, self.notify_url, ['a', 'b']))
         a = listener.notifier('pub-a', topics=['a'])
         b = listener.notifier('pub-b', topics=['b'])
 
@@ -363,7 +404,7 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
 
         received = {}
         while len(received) < len(sent):
-            e = listener.events.get(timeout=1)
+            e = listener.events.get(timeout=get_timeout)
             received[e[3]] = e
 
         for key in received:
@@ -374,18 +415,23 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
             self.assertEqual(expected[2], actual[2])
 
     def test_multiple_servers(self):
-        if self.url.startswith("amqp:"):
+        timeout = 0.5
+        if self.notify_url.startswith("amqp:"):
             self.skipTest("QPID-6307")
-        if self.url.startswith("zmq"):
-            self.skipTest("ZeroMQ-PUB-SUB")
-        if self.url.startswith("kafka"):
-            self.skipTest("Kafka: Need to be fixed")
+        if self.notify_url.startswith("kafka://"):
+            self.skipTest("Kafka: needs to be fixed")
+            timeout = 5
+            self.conf.set_override('consumer_group',
+                                   'test_multiple_servers',
+                                   group='oslo_messaging_kafka')
 
         listener_a = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['test-topic']))
+            utils.NotificationFixture(self.conf, self.notify_url,
+                                      ['test-topic']))
 
         listener_b = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['test-topic']))
+            utils.NotificationFixture(self.conf, self.notify_url,
+                                      ['test-topic']))
 
         n = listener_a.notifier('pub')
 
@@ -393,27 +439,29 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
         for event_type, payload in events_out:
             n.info({}, event_type, payload)
 
-        events_in = [[(e[1], e[2]) for e in listener_a.get_events()],
-                     [(e[1], e[2]) for e in listener_b.get_events()]]
+        events_in = [[(e[1], e[2]) for e in listener_a.get_events(timeout)],
+                     [(e[1], e[2]) for e in listener_b.get_events(timeout)]]
 
         self.assertThat(events_in, utils.IsValidDistributionOf(events_out))
         for stream in events_in:
             self.assertThat(len(stream), matchers.GreaterThan(0))
 
     def test_independent_topics(self):
-        if self.url.startswith("kafka://"):
+        get_timeout = 0.5
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 5
             self.conf.set_override('consumer_group',
                                    'test_independent_topics_a',
                                    group='oslo_messaging_kafka')
         listener_a = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['1']))
+            utils.NotificationFixture(self.conf, self.notify_url, ['1']))
 
-        if self.url.startswith("kafka://"):
+        if self.notify_url.startswith("kafka://"):
             self.conf.set_override('consumer_group',
                                    'test_independent_topics_b',
                                    group='oslo_messaging_kafka')
         listener_b = self.useFixture(
-            utils.NotificationFixture(self.conf, self.url, ['2']))
+            utils.NotificationFixture(self.conf, self.notify_url, ['2']))
 
         a = listener_a.notifier('pub-1', topics=['1'])
         b = listener_b.notifier('pub-2', topics=['2'])
@@ -427,7 +475,7 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
             b.info({}, event_type, payload)
 
         def check_received(listener, publisher, messages):
-            actuals = sorted([listener.events.get(timeout=0.5)
+            actuals = sorted([listener.events.get(timeout=get_timeout)
                               for __ in range(len(a_out))])
             expected = sorted([['info', m[0], m[1], publisher]
                                for m in messages])
@@ -437,12 +485,14 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
         check_received(listener_b, "pub-2", b_out)
 
     def test_all_categories(self):
-        if self.url.startswith("kafka://"):
+        get_timeout = 1
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 5
             self.conf.set_override('consumer_group', 'test_all_categories',
                                    group='oslo_messaging_kafka')
 
         listener = self.useFixture(utils.NotificationFixture(
-            self.conf, self.url, ['test_all_categories']))
+            self.conf, self.notify_url, ['test_all_categories']))
         n = listener.notifier('abc')
 
         cats = ['debug', 'audit', 'info', 'warn', 'error', 'critical']
@@ -453,7 +503,7 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
         # order between events with different categories is not guaranteed
         received = {}
         for expected in events:
-            e = listener.events.get(timeout=1)
+            e = listener.events.get(timeout=get_timeout)
             received[e[0]] = e
 
         for expected in events:
@@ -463,26 +513,55 @@ class NotifyTestCase(utils.SkipIfNoTransportURL):
             self.assertEqual(expected[3], actual[2])
 
     def test_simple_batch(self):
-        if self.url.startswith("amqp:"):
+        get_timeout = 3
+        batch_timeout = 2
+        if self.notify_url.startswith("amqp:"):
             backend = os.environ.get("AMQP1_BACKEND")
             if backend == "qdrouterd":
                 # end-to-end acknowledgement with router intermediary
                 # sender pends until batch_size or timeout reached
                 self.skipTest("qdrouterd backend")
-        if self.url.startswith("kafka://"):
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 10
+            batch_timeout = 5
             self.conf.set_override('consumer_group', 'test_simple_batch',
                                    group='oslo_messaging_kafka')
 
         listener = self.useFixture(
-            utils.BatchNotificationFixture(self.conf, self.url,
+            utils.BatchNotificationFixture(self.conf, self.notify_url,
                                            ['test_simple_batch'],
-                                           batch_size=100, batch_timeout=2))
+                                           batch_size=100,
+                                           batch_timeout=batch_timeout))
         notifier = listener.notifier('abc')
 
-        for i in six.moves.range(0, 205):
+        for i in range(0, 205):
             notifier.info({}, 'test%s' % i, 'Hello World!')
-        events = listener.get_events(timeout=3)
+        events = listener.get_events(timeout=get_timeout)
         self.assertEqual(3, len(events))
         self.assertEqual(100, len(events[0][1]))
         self.assertEqual(100, len(events[1][1]))
         self.assertEqual(5, len(events[2][1]))
+
+    def test_compression(self):
+        get_timeout = 1
+        if self.notify_url.startswith("amqp:"):
+            self.conf.set_override('kombu_compression', 'gzip',
+                                   group='oslo_messaging_rabbit')
+        if self.notify_url.startswith("kafka://"):
+            get_timeout = 5
+            self.conf.set_override('compression_codec', 'gzip',
+                                   group='oslo_messaging_kafka')
+            self.conf.set_override('consumer_group', 'test_compression',
+                                   group='oslo_messaging_kafka')
+
+        listener = self.useFixture(
+            utils.NotificationFixture(self.conf, self.notify_url,
+                                      ['test_compression']))
+        notifier = listener.notifier('abc')
+
+        notifier.info({}, 'test', 'Hello World!')
+        event = listener.events.get(timeout=get_timeout)
+        self.assertEqual('info', event[0])
+        self.assertEqual('test', event[1])
+        self.assertEqual('Hello World!', event[2])
+        self.assertEqual('abc', event[3])
