@@ -23,7 +23,7 @@ import testscenarios
 
 import oslo_messaging
 from oslo_messaging.tests import utils as test_utils
-from six.moves import mock
+from unittest import mock
 
 
 load_tests = testscenarios.load_tests_apply_scenarios
@@ -51,8 +51,7 @@ class TestLogNotifier(test_utils.BaseTestCase):
         self.config(driver=['test'],
                     group='oslo_messaging_notifications')
         # NOTE(jamespage) disable thread information logging for testing
-        # as this causes test failures when zmq tests monkey_patch via
-        # eventlet
+        # as this can cause test failures when monkey_patch via eventlet
         logging.logThreads = 0
 
     @mock.patch('oslo_utils.timeutils.utcnow')
