@@ -34,18 +34,17 @@ from oslo_messaging.notify import messaging
 from oslo_messaging.notify import notifier as msg_notifier
 from oslo_messaging import serializer as msg_serializer
 from oslo_messaging.tests import utils as test_utils
-from six.moves import mock
+from unittest import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class _FakeTransport(object):
+class JsonMessageMatcher(object):
+    def __init__(self, message):
+        self.message = message
 
-    def __init__(self, conf):
-        self.conf = conf
-
-    def _send_notification(self, target, ctxt, message, version, retry=None):
-        pass
+    def __eq__(self, other):
+        return self.message == jsonutils.loads(other)
 
 
 class _ReRaiseLoggedExceptionsFixture(fixtures.Fixture):
@@ -65,6 +64,9 @@ class _ReRaiseLoggedExceptionsFixture(fixtures.Fixture):
         def exception(self, msg, *args, **kwargs):
             self.exceptions.append(sys.exc_info()[1])
 
+        def warning(self, msg, *args, **kwargs):
+            return
+
     def setUp(self):
         super(_ReRaiseLoggedExceptionsFixture, self).setUp()
 
@@ -145,8 +147,10 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
         super(TestMessagingNotifier, self).setUp()
 
         self.logger = self.useFixture(_ReRaiseLoggedExceptionsFixture()).logger
-        self.stubs.Set(messaging, 'LOG', self.logger)
-        self.stubs.Set(msg_notifier, '_LOG', self.logger)
+        self.useFixture(fixtures.MockPatchObject(
+            messaging, 'LOG', self.logger))
+        self.useFixture(fixtures.MockPatchObject(
+            msg_notifier, '_LOG', self.logger))
 
     @mock.patch('oslo_utils.timeutils.utcnow')
     def test_notifier(self, mock_utcnow):
@@ -160,7 +164,8 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
                     topics=self.topics,
                     group='oslo_messaging_notifications')
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         if hasattr(self, 'ctor_pub_id'):
             notifier = oslo_messaging.Notifier(transport,
@@ -176,11 +181,10 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
         if prepare_kwds:
             notifier = notifier.prepare(**prepare_kwds)
 
-        self.mox.StubOutWithMock(transport, '_send_notification')
+        transport._send_notification = mock.Mock()
 
         message_id = uuid.uuid4()
-        self.mox.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(message_id)
+        uuid.uuid4 = mock.Mock(return_value=message_id)
 
         mock_utcnow.return_value = datetime.datetime.utcnow()
 
@@ -199,22 +203,28 @@ class TestMessagingNotifier(test_utils.BaseTestCase):
         if self.v2:
             sends.append(dict(version=2.0))
 
+        calls = []
         for send_kwargs in sends:
             for topic in self.topics:
                 if hasattr(self, 'retry'):
                     send_kwargs['retry'] = self.retry
                 else:
-                    send_kwargs['retry'] = None
+                    send_kwargs['retry'] = -1
                 target = oslo_messaging.Target(topic='%s.%s' % (topic,
                                                                 self.priority))
-                transport._send_notification(target, self.ctxt, message,
-                                             **send_kwargs).InAnyOrder()
-
-        self.mox.ReplayAll()
+                calls.append(mock.call(target,
+                                       self.ctxt,
+                                       message,
+                                       **send_kwargs))
 
         method = getattr(notifier, self.priority)
         method(self.ctxt, 'test.notify', self.payload)
 
+        uuid.uuid4.assert_called_once_with()
+        transport._send_notification.assert_has_calls(calls, any_order=True)
+
+        self.assertTrue(notifier.is_enabled())
+
 
 TestMessagingNotifier.generate_scenarios()
 
@@ -227,29 +237,27 @@ class TestSerializer(test_utils.BaseTestCase):
 
     @mock.patch('oslo_utils.timeutils.utcnow')
     def test_serializer(self, mock_utcnow):
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         serializer = msg_serializer.NoOpSerializer()
 
         notifier = oslo_messaging.Notifier(transport,
                                            'test.localhost',
                                            driver='test',
-                                           topic='test',
+                                           topics=['test'],
                                            serializer=serializer)
 
         message_id = uuid.uuid4()
-        self.mox.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(message_id)
+        uuid.uuid4 = mock.Mock(return_value=message_id)
 
         mock_utcnow.return_value = datetime.datetime.utcnow()
 
-        self.mox.StubOutWithMock(serializer, 'serialize_context')
-        self.mox.StubOutWithMock(serializer, 'serialize_entity')
-        serializer.serialize_context(dict(user='bob')).\
-            AndReturn(dict(user='alice'))
-        serializer.serialize_entity(dict(user='bob'), 'bar').AndReturn('sbar')
+        serializer.serialize_context = mock.Mock()
+        serializer.serialize_context.return_value = dict(user='alice')
 
-        self.mox.ReplayAll()
+        serializer.serialize_entity = mock.Mock()
+        serializer.serialize_entity.return_value = 'sbar'
 
         notifier.info(dict(user='bob'), 'test.notify', 'bar')
 
@@ -262,9 +270,14 @@ class TestSerializer(test_utils.BaseTestCase):
             'timestamp': str(timeutils.utcnow()),
         }
 
-        self.assertEqual([(dict(user='alice'), message, 'INFO', None)],
+        self.assertEqual([(dict(user='alice'), message, 'INFO', -1)],
                          _impl_test.NOTIFICATIONS)
 
+        uuid.uuid4.assert_called_once_with()
+        serializer.serialize_context.assert_called_once_with(dict(user='bob'))
+        serializer.serialize_entity.assert_called_once_with(dict(user='bob'),
+                                                            'bar')
+
 
 class TestNotifierTopics(test_utils.BaseTestCase):
 
@@ -273,7 +286,8 @@ class TestNotifierTopics(test_utils.BaseTestCase):
                     group='oslo_messaging_notifications')
         self.config(topics=['topic1', 'topic2'],
                     group='oslo_messaging_notifications')
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         notifier = oslo_messaging.Notifier(transport, 'test.localhost')
         self.assertEqual(['topic1', 'topic2'], notifier._topics)
@@ -281,11 +295,9 @@ class TestNotifierTopics(test_utils.BaseTestCase):
     def test_topics_from_kwargs(self):
         self.config(driver=['log'],
                     group='oslo_messaging_notifications')
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
-        notifier = oslo_messaging.Notifier(transport, 'test.localhost',
-                                           topic='topic1')
-        self.assertEqual(['topic1'], notifier._topics)
         notifier = oslo_messaging.Notifier(transport, 'test.localhost',
                                            topics=['topic1', 'topic2'])
         self.assertEqual(['topic1', 'topic2'], notifier._topics)
@@ -298,16 +310,19 @@ class TestLogNotifier(test_utils.BaseTestCase):
         self.config(driver=['log'],
                     group='oslo_messaging_notifications')
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         notifier = oslo_messaging.Notifier(transport, 'test.localhost')
 
         message_id = uuid.uuid4()
-        self.mox.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(message_id)
+        uuid.uuid4 = mock.Mock()
+        uuid.uuid4.return_value = message_id
 
         mock_utcnow.return_value = datetime.datetime.utcnow()
 
+        logger = mock.Mock()
+
         message = {
             'message_id': str(message_id),
             'publisher_id': 'test.localhost',
@@ -317,33 +332,36 @@ class TestLogNotifier(test_utils.BaseTestCase):
             'timestamp': str(timeutils.utcnow()),
         }
 
-        logger = self.mox.CreateMockAnything()
+        with mock.patch.object(logging, 'getLogger') as gl:
+            gl.return_value = logger
 
-        self.mox.StubOutWithMock(logging, 'getLogger')
-        logging.getLogger('oslo.messaging.notification.test.notify').\
-            AndReturn(logger)
+            notifier.info({}, 'test.notify', 'bar')
 
-        logger.info(jsonutils.dumps(message))
+            uuid.uuid4.assert_called_once_with()
+            logging.getLogger.assert_called_once_with(
+                'oslo.messaging.notification.test.notify')
 
-        self.mox.ReplayAll()
+        logger.info.assert_called_once_with(JsonMessageMatcher(message))
 
-        notifier.info({}, 'test.notify', 'bar')
+        self.assertTrue(notifier.is_enabled())
 
     def test_sample_priority(self):
         # Ensure logger drops sample-level notifications.
         driver = _impl_log.LogDriver(None, None, None)
 
-        logger = self.mox.CreateMock(
-            logging.getLogger('oslo.messaging.notification.foo'))
+        logger = mock.Mock(spec=logging.getLogger('oslo.messaging.'
+                                                  'notification.foo'))
         logger.sample = None
-        self.mox.StubOutWithMock(logging, 'getLogger')
-        logging.getLogger('oslo.messaging.notification.foo').\
-            AndReturn(logger)
-
-        self.mox.ReplayAll()
 
         msg = {'event_type': 'foo'}
-        driver.notify(None, msg, "sample", None)
+
+        with mock.patch.object(logging, 'getLogger') as gl:
+            gl.return_value = logger
+
+            driver.notify(None, msg, "sample", None)
+
+            logging.getLogger.assert_called_once_with('oslo.messaging.'
+                                                      'notification.foo')
 
     def test_mask_passwords(self):
         # Ensure that passwords are masked with notifications
@@ -360,16 +378,46 @@ class TestLogNotifier(test_utils.BaseTestCase):
         logger.info.assert_called_once_with(mask_str)
 
 
+class TestNotificationConfig(test_utils.BaseTestCase):
+
+    def test_retry_config(self):
+        conf = self.messaging_conf.conf
+        self.config(driver=['messaging'],
+                    group='oslo_messaging_notifications')
+
+        conf.set_override('retry', 3, group='oslo_messaging_notifications')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
+        notifier = oslo_messaging.Notifier(transport)
+
+        self.assertEqual(3, notifier.retry)
+
+    def test_notifier_retry_config(self):
+        conf = self.messaging_conf.conf
+        self.config(driver=['messaging'],
+                    group='oslo_messaging_notifications')
+
+        conf.set_override('retry', 3, group='oslo_messaging_notifications')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
+        notifier = oslo_messaging.Notifier(transport, retry=5)
+
+        self.assertEqual(5, notifier.retry)
+
+
 class TestRoutingNotifier(test_utils.BaseTestCase):
     def setUp(self):
         super(TestRoutingNotifier, self).setUp()
         self.config(driver=['routing'],
                     group='oslo_messaging_notifications')
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
         self.notifier = oslo_messaging.Notifier(transport)
         self.router = self.notifier._driver_mgr['routing'].obj
 
+        self.assertTrue(self.notifier.is_enabled())
+
     def _fake_extension_manager(self, ext):
         return extension.ExtensionManager.make_test_instance(
             [extension.Extension('test', None, None, ext), ])
@@ -426,10 +474,12 @@ group_2:
             with mock.patch('stevedore.dispatch.DispatchExtensionManager',
                             return_value=self._fake_extension_manager(
                                 mock.MagicMock())):
-                self.router._load_notifiers()
-                groups = list(self.router.routing_groups.keys())
-                groups.sort()
-                self.assertEqual(['group_1', 'group_2'], groups)
+                with mock.patch('oslo_messaging.notify.'
+                                '_impl_routing.LOG'):
+                    self.router._load_notifiers()
+                    groups = list(self.router.routing_groups.keys())
+                    groups.sort()
+                    self.assertEqual(['group_1', 'group_2'], groups)
 
     def test_get_drivers_for_message_accepted_events(self):
         config = r"""
@@ -578,9 +628,38 @@ group_1:
                                config_file):
             with mock.patch('stevedore.dispatch.DispatchExtensionManager',
                             return_value=pm):
-                self.notifier.info({}, 'my_event', {})
-                self.assertFalse(bar_driver.info.called)
-                rpc_driver.notify.assert_called_once_with(
-                    {}, mock.ANY, 'INFO', None)
-                rpc2_driver.notify.assert_called_once_with(
-                    {}, mock.ANY, 'INFO', None)
+                with mock.patch('oslo_messaging.notify.'
+                                '_impl_routing.LOG'):
+
+                    self.notifier.info({}, 'my_event', {})
+                    self.assertFalse(bar_driver.info.called)
+                    rpc_driver.notify.assert_called_once_with(
+                        {}, mock.ANY, 'INFO', -1)
+                    rpc2_driver.notify.assert_called_once_with(
+                        {}, mock.ANY, 'INFO', -1)
+
+
+class TestNoOpNotifier(test_utils.BaseTestCase):
+
+    def test_notifier(self):
+        self.config(driver=['noop'],
+                    group='oslo_messaging_notifications')
+
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
+
+        notifier = oslo_messaging.Notifier(transport, 'test.localhost')
+
+        self.assertFalse(notifier.is_enabled())
+
+
+class TestNotifierTransportWarning(test_utils.BaseTestCase):
+
+    @mock.patch('oslo_messaging.notify.notifier._LOG')
+    def test_warning_when_rpc_transport(self, log):
+        transport = oslo_messaging.get_rpc_transport(self.conf)
+        oslo_messaging.Notifier(transport, 'test.localhost')
+        log.warning.assert_called_once_with(
+            "Using RPC transport for notifications. Please use "
+            "get_notification_transport to obtain a "
+            "notification transport instance.")
