@@ -16,7 +16,6 @@
 import sys
 
 from oslo_serialization import jsonutils
-import six
 import testscenarios
 
 import oslo_messaging
@@ -25,7 +24,8 @@ from oslo_messaging.tests import utils as test_utils
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
-EXCEPTIONS_MODULE = 'exceptions' if six.PY2 else 'builtins'
+EXCEPTIONS_MODULE = 'builtins'
+OTHER_EXCEPTIONS_MODULE = 'exceptions'
 
 
 class NovaStyleException(Exception):
@@ -98,9 +98,6 @@ class SerializeRemoteExceptionTestCase(test_utils.BaseTestCase):
         cls.scenarios = testscenarios.multiply_scenarios(cls._add_remote,
                                                          cls._exception_types)
 
-    def setUp(self):
-        super(SerializeRemoteExceptionTestCase, self).setUp()
-
     def test_serialize_remote_exception(self):
         try:
             try:
@@ -149,6 +146,17 @@ class DeserializeRemoteExceptionTestCase(test_utils.BaseTestCase):
               remote_name='Exception',
               remote_args=('test\ntraceback\ntraceback\n', ),
               remote_kwargs={})),
+        ('different_python_versions',
+         dict(allowed=_standard_allowed,
+              clsname='Exception',
+              modname=OTHER_EXCEPTIONS_MODULE,
+              cls=Exception,
+              args=['test'],
+              kwargs={},
+              str='test\ntraceback\ntraceback\n',
+              remote_name='Exception',
+              remote_args=('test\ntraceback\ntraceback\n', ),
+              remote_kwargs={})),
         ('nova_style',
          dict(allowed=_standard_allowed,
               clsname='NovaStyleException',
@@ -280,9 +288,9 @@ class DeserializeRemoteExceptionTestCase(test_utils.BaseTestCase):
 
         self.assertIsInstance(ex, self.cls)
         self.assertEqual(self.remote_name, ex.__class__.__name__)
-        self.assertEqual(self.str, six.text_type(ex))
+        self.assertEqual(self.str, str(ex))
         if hasattr(self, 'msg'):
-            self.assertEqual(self.msg, six.text_type(ex))
+            self.assertEqual(self.msg, str(ex))
             self.assertEqual((self.msg,) + self.remote_args, ex.args)
         else:
             self.assertEqual(self.remote_args, ex.args)
