@@ -12,11 +12,12 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+from unittest import mock
+
 import stevedore
 import testtools
 
-import mock
-
 from oslo_messaging import server
 try:
     from oslo_messaging import opts
@@ -25,25 +26,18 @@ except ImportError:
 from oslo_messaging.tests import utils as test_utils
 
 
+@testtools.skipIf(opts is None, "Options not importable")
 class OptsTestCase(test_utils.BaseTestCase):
 
-    @testtools.skipIf(opts is None, "Options not importable")
-    def setUp(self):
-        super(OptsTestCase, self).setUp()
-
     def _test_list_opts(self, result):
-        self.assertEqual(6, len(result))
+        self.assertEqual(5, len(result))
 
         groups = [g for (g, l) in result]
         self.assertIn(None, groups)
-        self.assertIn('matchmaker_redis', groups)
-        self.assertIn('oslo_messaging_zmq', groups)
         self.assertIn('oslo_messaging_amqp', groups)
         self.assertIn('oslo_messaging_notifications', groups)
         self.assertIn('oslo_messaging_rabbit', groups)
-
-        opt_names = [o.name for (g, l) in result for o in l]
-        self.assertIn('rpc_backend', opt_names)
+        self.assertIn('oslo_messaging_kafka', groups)
 
     def test_list_opts(self):
         self._test_list_opts(opts.list_opts())
