@@ -17,31 +17,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import logging
+
+from debtcollector import removals
+from oslo_config import cfg
+from stevedore import driver
+from urllib import parse
+
+from oslo_messaging import exceptions
+
 __all__ = [
     'DriverLoadFailure',
     'InvalidTransportURL',
     'Transport',
     'TransportHost',
     'TransportURL',
+    'TransportOptions',
     'get_transport',
     'set_transport_defaults',
 ]
 
-import logging
-
-from debtcollector import removals
-from oslo_config import cfg
-import six
-from six.moves.urllib import parse
-from stevedore import driver
-
-from oslo_messaging._i18n import _LW
-from oslo_messaging import exceptions
-
 LOG = logging.getLogger(__name__)
 
 _transport_opts = [
     cfg.StrOpt('transport_url',
+               default="rabbit://",
                secret=True,
                help='The network address and optional user credentials for '
                     'connecting to the messaging backend, in URL format. The '
@@ -53,12 +53,6 @@ _transport_opts = [
                     'documentation of oslo_messaging.TransportURL at '
                     'https://docs.openstack.org/oslo.messaging/latest/'
                     'reference/transport.html'),
-    cfg.StrOpt('rpc_backend',
-               deprecated_for_removal=True,
-               deprecated_reason="Replaced by [DEFAULT]/transport_url",
-               default='rabbit',
-               help='The messaging driver to use, defaults to rabbit. Other '
-                    'drivers include amqp and zmq.'),
 
     cfg.StrOpt('control_exchange',
                default='openstack',
@@ -122,7 +116,7 @@ class Transport(object):
         self._driver.require_features(requeue=requeue)
 
     def _send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
-              call_monitor_timeout=None, retry=None):
+              call_monitor_timeout=None, retry=None, transport_options=None):
         if not target.topic:
             raise exceptions.InvalidTarget('A topic is required to send',
                                            target)
@@ -130,7 +124,8 @@ class Transport(object):
                                  wait_for_reply=wait_for_reply,
                                  timeout=timeout,
                                  call_monitor_timeout=call_monitor_timeout,
-                                 retry=retry)
+                                 retry=retry,
+                                 transport_options=transport_options)
 
     def _send_notification(self, target, ctxt, message, version, retry=None):
         if not target.topic:
@@ -282,6 +277,16 @@ class TransportHost(object):
         return '<TransportHost ' + values + '>'
 
 
+class TransportOptions(object):
+
+    def __init__(self, at_least_once=False):
+        self._at_least_once = at_least_once
+
+    @property
+    def at_least_once(self):
+        return self._at_least_once
+
+
 class TransportURL(object):
 
     """A parsed transport URL.
@@ -339,7 +344,7 @@ class TransportURL(object):
                  query=None):
         self.conf = conf
         self.conf.register_opts(_transport_opts)
-        self._transport = transport
+        self.transport = transport
         self.virtual_host = virtual_host
         if hosts is None:
             self.hosts = []
@@ -350,33 +355,6 @@ class TransportURL(object):
         else:
             self.query = query
 
-        self._deprecation_logged = False
-
-    @property
-    def transport(self):
-        if self._transport is None:
-            transport = self.conf.rpc_backend
-        else:
-            transport = self._transport
-        final_transport = transport
-        if not self._deprecation_logged and final_transport != transport:
-            # NOTE(sileht): The first step is deprecate this one cycle.
-            # To ensure deployer have updated they configuration during Ocata
-            # Then in P we will deprecate aliases kwargs of TransportURL() and
-            # get_transport() for consuming application
-            LOG.warning('legacy "rpc_backend" is deprecated, '
-                        '"%(legacy_transport)s" must be replaced by '
-                        '"%(final_transport)s"' % {
-                            'legacy_transport': transport,
-                            'final_transport': final_transport})
-            self._deprecation_logged = True
-
-        return final_transport
-
-    @transport.setter
-    def transport(self, value):
-        self._transport = value
-
     def __hash__(self):
         return hash((tuple(self.hosts), self.transport, self.virtual_host))
 
@@ -476,10 +454,8 @@ class TransportURL(object):
         if not url:
             conf.register_opts(_transport_opts)
         url = url or conf.transport_url
-        if not url:
-            return cls(conf)
 
-        if not isinstance(url, six.string_types):
+        if not isinstance(url, str):
             raise InvalidTransportURL(url, 'Wrong URL type')
 
         url = parse.urlparse(url)
@@ -554,10 +530,10 @@ class TransportURL(object):
 
         if (len(hosts_with_credentials) > 0 and
                 len(hosts_without_credentials) > 0):
-            LOG.warning(_LW("All hosts must be set with username/password or "
-                            "not at the same time. Hosts with credentials "
-                            "are: %(hosts_with_credentials)s. Hosts without "
-                            "credentials are %(hosts_without_credentials)s."),
+            LOG.warning("All hosts must be set with username/password or "
+                        "not at the same time. Hosts with credentials "
+                        "are: %(hosts_with_credentials)s. Hosts without "
+                        "credentials are %(hosts_without_credentials)s.",
                         {'hosts_with_credentials': hosts_with_credentials,
                          'hosts_without_credentials':
                          hosts_without_credentials})
